/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.commons.log;

import javax.jbi.messaging.MessageExchange;

/**
 * This class provides facilities for storing the flow attributes state into a message exchange.
 * 
 * Note that the object of class {@link FlowAttributes} is not stored itself to prevent classloading issues as the data
 * will be shared by multiple actors engaged in an exchange and they only share the JBI classloader which does not know
 * about {@link FlowAttributes}.
 *
 * @author vnoel
 *
 */
public class FlowAttributesExchangeHelper {

    public static final String DEFAULT_PREFIX = "org.ow2.petals.commons.log.";

    public static final String FLOW_INSTANCE_ID = "FlowAttributesInstanceId";

    public static final String FLOW_STEP_ID = "FlowAttributesStepId";

    private FlowAttributesExchangeHelper() {
    }

    public static FlowAttributes getFlowAttributes(final MessageExchange messageExchange, final String prefix) {
        final Object oI = messageExchange.getProperty(prefix + FLOW_INSTANCE_ID);
        final Object oS = messageExchange.getProperty(prefix + FLOW_STEP_ID);
        if (oS instanceof String && oI instanceof String) {
            return new FlowAttributes((String) oI, (String) oS);
        } else {
            return null;
        }
    }

    /**
     * Set the flow instance identifier and the flow step identifier from the given flow attributes into the given
     * exchange as properties
     * 
     * @param messageExchange
     *            The message exchange in which flow information must be set
     * @param attributes
     *            The flow attribute containing the flow information to set
     * @param prefix
     *            A prefix used in the property name
     */
    public static void setFlowAttributes(final MessageExchange messageExchange, final FlowAttributes attributes,
            final String prefix) {
        if (attributes != null) {
            messageExchange.setProperty(prefix + FLOW_INSTANCE_ID, attributes.getFlowInstanceId());
            messageExchange.setProperty(prefix + FLOW_STEP_ID, attributes.getFlowStepId());
        }
    }

    public static FlowAttributes getFlowAttributes(final MessageExchange messageExchange) {
        return getFlowAttributes(messageExchange, DEFAULT_PREFIX);
    }

    /**
     * <p>
     * Set the flow instance identifier and the flow step identifier from the given flow attributes into the given
     * exchange as properties.
     * </p>
     * <p>
     * Note: property name is prefixed with the default prefix {@link #DEFAULT_PREFIX}.
     * </p>
     * 
     * @param messageExchange
     *            The message exchange in which flow information must be set
     * @param attributes
     *            The flow attribute containing the flow information to set
     */
    public static void setFlowAttributes(final MessageExchange messageExchange, final FlowAttributes attributes) {
        setFlowAttributes(messageExchange, attributes, DEFAULT_PREFIX);
    }

}
