/**
 * Copyright (c) 2011-2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.commons.log;

import com.ebmwebsourcing.easycommons.thread.ExecutionContext;
import com.ebmwebsourcing.easycommons.uuid.SimpleUUIDGenerator;

/**
 * 
 * This contains everything needed to store informations about the context of execution of a flow.
 * 
 * It is stored in a thread local ({@link ExecutionContext}) in order for the logging infrastructure (MONIT traces) to
 * access this information, in the same manner of the MDC from log4j, logtalk, slf4j, etc.
 * 
 * See also the project petals-cdk-steplog and petals-log that access this information.
 * 
 * Note: some of the properties declared here are used as keys of subclasses of FlowLogData (in petals-cdk-steplog) that
 * are not visible (in terms of classpath) to the users of these properties (for example in petals-log, that do not
 * depends on petals-cdk-steplog): that's why they are here and not declared in the class that exploits them. This is
 * not the cleaner way to do that.
 * 
 * TODO maybe move all the LogData things to petals-cdk-steplog and keep here only LoggerHelper, PetalsExecutionContext
 * and FlowAttributes helpers?
 * 
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public final class PetalsExecutionContext {

    /**
     * Name of the contextual property storing the container name
     */
    static final String CONTAINER_NAME_PROPERTY_NAME = "containerName";

    /**
     * Name of the contextual property (printed in MONIT trace) storing the flow
     * instance identifier. A flow instance is made up of flow steps.
     */
    static final String FLOW_INSTANCE_ID_PROPERTY_NAME = "flowInstanceId";

    /**
     * Name of the contextual property (printed in MONIT trace) storing the flow
     * step identifier of the flow step in the current flow instance.
     */
    static final String FLOW_STEP_ID_PROPERTY_NAME = "flowStepId";

    /**
     * An UUID generator.
     */
    private static final SimpleUUIDGenerator simpleUUIDGenerator = new SimpleUUIDGenerator();

    /**
     * Set in the {@link ExecutionContext}, the value of
     * {@link PetalsExecutionContext#CONTAINER_NAME_PROPERTY_NAME}.
     * 
     * @param containerName
     *            the container name to set as value of the contextual property
     *            {@link PetalsExecutionContext#CONTAINER_NAME_PROPERTY_NAME}.
     */
    public static void putContainerName(final String containerName) {
        assert containerName != null;
        ExecutionContext.getProperties().setProperty(CONTAINER_NAME_PROPERTY_NAME, containerName);
    }

    public static String getContainerName() {
        return ExecutionContext.getProperties().getProperty(CONTAINER_NAME_PROPERTY_NAME);
    }

    /**
     * Set in the {@link ExecutionContext}, the value of
     * {@link PetalsExecutionContext#FLOW_INSTANCE_ID_PROPERTY_NAME}.
     * 
     * @param flowInstanceId
     *            the flow instance identifier to set as value of the contextual
     *            property
     *            {@link PetalsExecutionContext#FLOW_INSTANCE_ID_PROPERTY_NAME}.
     */
    static void putFlowInstanceId(String flowInstanceId) {
        if (flowInstanceId != null) {
            ExecutionContext.getProperties().setProperty(FLOW_INSTANCE_ID_PROPERTY_NAME, flowInstanceId);
        } else {
            ExecutionContext.getProperties().remove(FLOW_INSTANCE_ID_PROPERTY_NAME);
        }
    }

    /**
     * Initialize the the value of
     * {@link PetalsExecutionContext#FLOW_INSTANCE_ID_PROPERTY_NAME}
     */
    public static FlowAttributes initFlowAttributes() {
        putFlowInstanceId(simpleUUIDGenerator.getNewID());
        putFlowStepId(simpleUUIDGenerator.getNewID());
        return getFlowAttributes();
    }

    static void putFlowStepId(String flowStepId) {
        if (flowStepId != null) {
            ExecutionContext.getProperties().setProperty(FLOW_STEP_ID_PROPERTY_NAME, flowStepId);
        } else {
            ExecutionContext.getProperties().remove(FLOW_STEP_ID_PROPERTY_NAME);
        }
    }

    /**
     * Set previous and current stepId in the {@link ExecutionContext} Only
     * provider mode.
     */
    public static FlowAttributes nextFlowStepId() {
        final String flowInstanceId = getFlowInstanceId();

        // this only make sense if we are currently in a flow instance id
        if (flowInstanceId != null) {
            final String newFlowStepId = simpleUUIDGenerator.getNewID();
            putFlowStepId(newFlowStepId);
            return new FlowAttributes(flowInstanceId, newFlowStepId);
        }

        return null;
    }

    /**
     * Remove all properties except the container name
     */
    public static void clear() {
        final String containerName = ExecutionContext.getProperties().getProperty(CONTAINER_NAME_PROPERTY_NAME);
        ExecutionContext.getProperties().clear();
        if (containerName != null) {
            putContainerName(containerName);
        }
    }

    public static String getFlowInstanceId() {
        return ExecutionContext.getProperties().getProperty(FLOW_INSTANCE_ID_PROPERTY_NAME);
    }

    public static String getFlowStepId() {
        return ExecutionContext.getProperties().getProperty(FLOW_STEP_ID_PROPERTY_NAME);
    }

    public static FlowAttributes getFlowAttributes() {
        final String flowInstanceId = getFlowInstanceId();
        if (flowInstanceId == null) {
            return null;
        }
        final String flowStepId = getFlowStepId();
        if (flowStepId == null) {
            return null;
        }
        return new FlowAttributes(flowInstanceId, flowStepId);
    }

    public static void putFlowAttributes(FlowAttributes flowAttributes) {
        if (flowAttributes != null) {
            putFlowInstanceId(flowAttributes.getFlowInstanceId());
            putFlowStepId(flowAttributes.getFlowStepId());
        } else {
            putFlowInstanceId(null);
            putFlowStepId(null);
        }
    }
}
